# -*- coding: utf-8 -*-
from bs4 import BeautifulSoup
from bs4 import SoupStrainer
import urllib
import urllib2
import nltk
import string
from collections import Counter
import math
import os
from openpyxl import load_workbook
from openpyxl import Workbook
import re
from textstat.textstat import textstat
from textblob import TextBlob
import numpy as np
import requests
import elasticsearch
from itertools import chain
from nltk import ngrams
from langdetect import detect


###########
#FUNCTIONS#
###########
ascii = set(string.printable)
def remove_non_ascii(s):
    return filter(lambda x: x in ascii, s)


#############
#DEFINITIONS#
#############
Punctuation = [',','?',';',':',')','(','@','!','[',']','{','}','.','\\',' - ',' -','- ','/','%','*','^','+','<','>','=','|','`']
Punctuation2 = [',','?',';',':',')','(','@','!','[',']','{','}','.','\\',' - ',' -','- ','-','/','%','*','^','+','<','>','=','|','`']
Other_Punct = ['"',"'"]

#Flesch Reading Ease:
FRE = []
FRE[0:30] = (30 - 0) * ['Very Confusing'];FRE[30:50] = (50 - 30) * ['Difficult'];FRE[50:60] = (60 - 50) * ['Fairly Difficult'];FRE[60:70] = (70 - 60) * ['Standard'];FRE[70:80] = (80 - 70) * ['Fairly Easy'];FRE[80:90] = (90 - 80) * ['Easy'];FRE[90:110] = (110 - 90) * ['Very Easy']
FRE_R = []
FRE_R[0:30] = (30 - 0) * ['Very Confusing'];FRE_R[30:50] = (50 - 30) * ['Difficult'];FRE_R[50:60] = (60 - 50) * ['Fairly Difficult'];FRE_R[60:70] = (70 - 60) * ['Standard'];FRE_R[70:80] = (80 - 70) * ['Fairly Easy'];FRE_R[80:90] = (90 - 80) * ['Easy'];FRE_R[90:120] = (120 - 90) * ['Very Easy']

#SMOG grade level: # texts of fewer than 30 sentences are statistically invalid, because the formula was normed on 30-sentence samples.
SMOG = []
SMOG[0:6] = (6 - 0) * [''];SMOG[6:7] = (7 - 6) * ['Kindergarten'];SMOG[7:8] = (8 - 7) * ['1st Grade'];SMOG[8:9] = (9 - 8) * ['2nd Grade'];SMOG[9:10] = (10 - 9) * ['3rd Grade'];SMOG[10:11] = (11 - 10) * ['4th Grade'];SMOG[11:12] = (12 - 11) * ['5th Grade'];SMOG[12:13] = (13 - 12) * ['6th Grade'];SMOG[13:14] = (14 - 13) * ['7th Grade'];SMOG[14:15] = (15 - 14) * ['8th Grade'];SMOG[15:16] = (16 - 15) * ['9th Grade'];SMOG[16:17] = (17 - 16) * ['10th Grade'];SMOG[17:18] = (18 - 17) * ['11th Grade'];SMOG[18:19] = (19 - 18) * ['12th Grade'];SMOG[19:50] = (50 - 19) * ['College']

#Coleman Liau Grade level
COLEMAN = []
COLEMAN[0:1] = (1 - 0) * ['1'];COLEMAN[1:2] = (2 - 1) * ['2'];COLEMAN[2:3] = (3 - 2) * ['3'];COLEMAN[3:4] = (4 - 3) * ['4'];COLEMAN[4:5] = (5 - 4) * ['5'];COLEMAN[5:6] = (6 - 5) * ['6'];COLEMAN[6:7] = (7 - 6) * ['7'];COLEMAN[7:8] = (8 - 7) * ['8'];COLEMAN[8:9] = (9 - 8) * ['9'];COLEMAN[9:10] = (10 - 9) * ['10'];COLEMAN[10:11] = (11 - 10) * ['11'];COLEMAN[11:12] = (12 - 11) * ['12'];COLEMAN[12:50] = (50 - 12) * ['College']

#Gunning Fog:
GUNNING = []
GUNNING[0:6] = (6 - 0) * ['None'];GUNNING[6:7] = (7 - 6) * ['6th grade'];GUNNING[7:8] = (8 - 7) * ['7th Grade'];GUNNING[8:9] = (9 - 8) * ['8th Grade'];GUNNING[9:10] = (10 - 9) * ['High school freshman'];GUNNING[10:11] = (11 - 10) * ['High school sophomore'];GUNNING[11:12] = (12 - 11) * ['High school junior'];GUNNING[12:13] = (13 - 12) * ['High school senior'];GUNNING[13:14] = (14 - 13) * ['College freshman'];GUNNING[14:15] = (15 - 14) * ['College sophomore'];GUNNING[15:16] = (16 - 15) * ['College junior'];GUNNING[16:17] = (17 - 16) * ['College senior'];GUNNING[17:55] = (55 - 17) * ['College graduate']

#Automated readability index (ARI)
ARI = []
ARI_age = []
ARI[1:2] = (2 - 1) * ['Kindergarten'];ARI[2:3] = (3 - 2) * ['First'];ARI[3:4] = (4 - 3) * ['Second'];ARI[4:5] = (5 - 4) * ['Third'];ARI[5:6] = (6 - 5) * ['Fourth'];ARI[6:7] = (7 - 6) * ['Fifth'];ARI[7:8] = (8 - 7) * ['Sixth'];ARI[8:9] = (9 - 8) * ['Seventh'];ARI[9:10] = (10 - 9) * ['Eighth'];ARI[10:11] = (11 - 10) * ['Ninth'];ARI[11:12] = (12 - 11) * ['Tenth'];ARI[12:13] = (13 - 12) * ['Eleventh'];ARI[13:14] = (14 - 13) * ['Twelfth'];ARI[14:60] = (60 - 14) * ['College'];
ARI_age[1:2] = (2 - 1) * ['5-6'];ARI_age[2:3] = (3 - 2) * ['6-7'];ARI_age[3:4] = (4 - 3) * ['7-8'];ARI_age[4:5] = (5 - 4) * ['8-9'];ARI_age[5:6] = (6 - 5) * ['9-10'];ARI_age[6:7] = (7 - 6) * ['10-11'];ARI_age[7:8] = (8 - 7) * ['11-12'];ARI_age[8:9] = (9 - 8) * ['12-13'];ARI_age[9:10] = (10 - 9) * ['13-14'];ARI_age[10:11] = (11 - 10) * ['14-15'];ARI_age[11:12] = (12 - 11) * ['15-16'];ARI_age[12:13] = (13 - 12) * ['16-17'];ARI_age[13:14] = (14 - 13) * ['17-18'];ARI_age[14:60] = (60 - 14) * ['18+'];

#Dale-Chall:
DALE = []
DALE[0:5] = (5 - 0) * ['4th grade or lower']; DALE[5:6] = (6 - 5) * ['5th or 6th grade']; DALE[6:7] = (7 - 6) * ['7th or 8th grade']; DALE[7:8] = (8 - 7) * ['9th or 10th grade']; DALE[8:9] = (9 - 8) * ['11th or 12th grade']; DALE[9:11] = (11 - 9) * ['13th to 15th grade (college)']


syllable_fix_list_plus = ['people','being','able','centre','going','doing','simple','issue','client','clients','agree','trying','double','seeing','pursue',
                     'mallee','couple','couples','middle','degree','google','struggle','cycle','saddle','example','media','video','unable','nominee',
                          'applying','valuable','suppliers','applying','principle','newcastle','audio','audience','single','little','liable','double','debbie','sample','stable','metre',
                          'dazzle','ripple','decree','little','searchable','taxable','possible','previous','suitable','guardian','ongoing','flexible','multiple','payable','variable','portable',
                          'maybe','ankle','example','value','google','louise','eddie','theatre','rundle','coffee','joanna','supplier','stephanie','premiums','newcastle','crying',
                          'trouble','battle','circle','handle','fluid','poetry','whoever','visible','violate']

syllable_fix_list_minus = ['based','lives','used','aged','sites','types','ecei','rolled','asked','states','gives','makes','called','takes','roles','shared',
                           'planned','reached','aimed','passed','trained','learned','looked','phased','filled','formed','placed','themes','deemed','files',
                           'homes','notes','likely','detailed','accessed','engaged','safety','offices','covered','updates','statements','uesful','prepared',
                           'minutes','advised','compared','employed','facebook','opened','pressures','describes','defined','relates','welcomes','guideline',
                           'advanced','offered','allowed','figures','explained','preferred','combined','entered','noticed','realised','widely','decades',
                           'focussed','outcomes','managed','decides','becomes','arrived','arrives','believes','finished','couples','deferred','designed',
                           'divorced','revealed','includes','displayed','occurred','outlines','provides','released','unused','careful','solely','lifetime',
                           'barely','lonely','lovely','lifelong','schemes','closely','therefore','timely','safely','lifetime','lifetimes','largely','barely','merely',
                           'bases','acres','bytes','eyes','names','dipper','popped','includes','receives','focused','expires','resides','refused','themselves','ourselves',
                           'widowed']


nouns           = ["NN", "NNP", "NNPS", "NNS"]
verbs           = ["VB", "VBD", "VBG", "VBP", "VBZ"]
adjectives      = ["JJ", "JJR"]
adverbs         = ["RB", "RBR", "RBS", "WRB"]
pronouns        = ["PRP", "PRP$", "WP", "WP$"]
prepositions    = ["IN"]
determiners     = ["DT", "PDT", "WDT"]
interjections   = ["UH"]
proper_nouns    = ["NNP","NNPS"]

Pronouns = ["I","me","my","mine","myself","we","us","our","ours","ourselves","you","your","yours","yourself","yourselves","he","him","his",
            "himself","she","her","hers","herself","it","its","itself","they","them","their","theirs","themselves"]

#Cohesive Devices: https://englishpost.org/teaching-grammar-linking-words/
cohesive_words = ['and','also','too','equally','identically','similarly','first','then','in addition',
                  'finally','but','however','nevertheless','meanwhile','certainly','absolutely','obviously',
                  'definitely','again','besides','next','further','furthermore','nor','moreover','whereas',
                  'where','yet','although','because','for','indeed','evidently','second','third','next',
                  'now','after','afterward','afterwards','subsequently','consequently','previously',
                  'simultaneously','concurrently','therefore','hence','soon']



Columns = ['Organisation','Article Name','Section','Text','URL','Partial URL', 'Intended audience','Flesch Reading Ease numerical score','Flesch Reading Ease general score','Gunning Fog numerical score',
             'Gunning Fog general score','Flesch-Kincaid Grade Level detail','Flesch-Kincaid Grade Level general','Coleman-Liau Index','Coleman-Liau Grade level',
             'SMOG Index','SMOG grade level','Automated Readability Index','Automated Readability Age Span','Automated Readability Grade level','Linsear Write Formula',
             'Linsear Write grade level','Readability Consensus','Based on 8 readability formulas, Grade Level','Reading Level','Readers Age','Readers grade level','Word count','Total # of unique words',
             'unique words as % of total text','Total # of repeat words','repeat words as % of total text','Average # of words per sentence','Sentence count',
             'Character count','Average # of characters per word','Average # of syllables per word','Syllable count','2-Syllable words count',
             'Percent of double syllables in text','1-Syllable words count','Percent of single syllables in text','3+Syllable words count','Percent of 3+ syllables in text',
             'Polysyllabic Count','Monosyllabic Words','Duosyllabic Words','Polysyllabic Words','Total Pronoun Counts',"I","me","my","mine","myself","we","us","our","ours","ourselves","you","your","yours","yourself","yourselves","he","him","his",
             "himself","she","her","hers","herself","it","its","itself","they","them","their","theirs","themselves",
               "Pronoun Ratio","Pronoun Ratio Indicator","Pronouns Listing","Nouns","Verbs","Adjectives","Adverbs","Pronouns","Prepositions","Determiners","Interjections",
             "Adjectives 1 Count","Adjectives 1 words","Adjectives 2 Count","Adjectives 2 words","Adjectives 3 Count","Adjectives 3 words",
                "Adverbs 1 Count","Adverbs 1 words","Adverbs 2 Count","Adverbs 2 words","Adverbs 3 Count","Adverbs 3 words",
                "Determiners 1 Count","Determiners 1 words","Determiners 2 Count","Determiners 2 words","Determiners 3 Count","Determiners 3 words",
                "Interjections 1 Count","Interjections 1 words","Interjections 2 Count","Interjections 2 words","Interjections 3 Count","Interjections 3 words",
                "Nouns 1 Count","Nouns 1 words","Nouns 2 Count","Nouns 2 words","Nouns 3 Count","Nouns 3 words",
                "Prepositions 1 Count","Prepositions 1 words","Prepositions 2 Count","Prepositions 2 words","Prepositions 3 Count","Prepositions 3 words",
                "Pronouns 1 Count","Pronouns 1 words","Pronouns 2 Count","Pronouns 2 words","Pronouns 3 Count","Pronouns 3 words",
                "Verbs 1 Count","Verbs 1 words","Verbs 2 Count","Verbs 2 words","Verbs 3 Count","Verbs 3 words","Proper Nouns Count","Proper Nouns words",
               "SMOG 30+ sentences indicator","Cohesive Words Count","List Items Count"
                ]

folders = ['About Us','Abstudy Austudy','Assurance Support','Basics Card','Bereavement Allowance','Business Services','Carer','Centrepay','Child Care','Crisis Payment',
           'Dad and Partner Pay','Disability','Double Orphan','Early Release Superannuation','Electronic Messaging','Family Tax Benefit','Farm or Remote Area',
           'Health Care Card','Income Management','Isolated Children','Jobs Education','Mobility Allowance','Newborn Upfront','Newstart','Online Help','Other',
           'Other Centrelink Stuff','Parental Leave Pay','Pension','Relocation','Rent Assistance','Sickness Allowance','Single Income Family','Stillborn Baby Payment',
           'Student Start Loan','Telephone Allowance','Widow Allowance','Youth Allowance']



###############
#LINK IMPORTER#
###############
wb = load_workbook(filename = 'C:/Python27/Scripts/Centrelink/The Final Scrape/DHS URLS - March 11.xlsx')
ws = wb.active
rowcount = ws.max_row

link_list = []
failed_pages = []
success = []

for row in range(1,rowcount):
    if 'centrelink' in ws.cell(row=row, column=1).value:
        link_list.append(ws.cell(row=row, column=1).value)

###################
#DOWNLOAD WEBPAGES#
###################
def extract_source(url):
    headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:50.0) Gecko/20100101 Firefox/52.9'}
    source=requests.get(url, headers=headers).text
    return source

len(link_list)

no_dice = []
for this_page in link_list:
    try:
        response = extract_source(this_page)
        for_saving = 'C:/Python27/Scripts/Centrelink/The Final Scrape/Centrelink March 11/Pages/' + this_page[33:].replace('/','_') + '.txt'
        with open(for_saving, 'wb') as f:
            f.write(response.encode('utf-8'))
    except:
        no_dice.append(this_page)

    
################
#WORKBOOK SETUP#
################
wb = Workbook()
ws = wb.active
row = 2
column = 1
for i in Columns:
    ws.cell(row=1, column=column, value=i)
    column = column + 1

##########################
#TEXT EXTRACTOR & STORAGE#
##########################

row = 2     

documents = os.listdir("C:/Python27/Scripts/Centrelink/The Final Scrape/Centrelink March 11/Pages")

for doc in documents:
    r = urllib.urlopen("/Python27/Scripts/Centrelink/The Final Scrape/Centrelink March 11/Pages/"+str(doc)).read()
    soup = BeautifulSoup(r,"html.parser")
    soup.header.decompose()
    soup.footer.decompose()
    list_item_counter = 0
    main_part = soup.main
    if detect(main_part.text) == 'en':
        link = doc.replace('_','/').replace('.txt','') 
        try:
            title = soup.find('div', {"class":"sub-title"}).text + ' ' + soup.h1.text + '_' + str(row)
        except:
            title = soup.h1.text + '_' + str(row)
        try:
            for tag in range(0,len(soup.find_all('i', {"class":"icon-ok"}))):
                soup.find('i', {"class":"icon-ok"}).decompose()
        except:
            pass
        try:
            for tag in range(0,len(soup.find_all('i', {"class":"icon-cancel"}))):
                soup.find('i', {"class":"icon-cancel"}).decompose()
        except:
            pass  
        list_of_text_from_tables = []     #list text in tables. eg. https://www.humanservices.gov.au/individuals/enablers/paying-tax-centrelink-payment
        for p in range(0,len(soup.find_all('td'))):
            if soup.main.table.tbody.td:
                holding_table_text = soup.main.table.tbody.td.text.encode('utf-8')+'.'
                if '\n' in holding_table_text[:10]:
                    holding_table_text = holding_table_text[:10].replace('\n','') + holding_table_text[10:]
                if '\n' in holding_table_text[-10:]:
                    holding_table_text = holding_table_text[:-10] + holding_table_text[-10:].replace('\n','')
                table_text = holding_table_text.replace('\xe2\x80\x99','\'').replace('\xe2\x80\x93','-').replace('\xc2\xa0',' ').replace('\n',', ').replace(', , ',', ').replace(':,',':')
                soup.main.table.tbody.td.decompose()
                list_of_text_from_tables.append(table_text)
        try:
            for tag in range(0,len(soup.find_all('h2'))):
                #soup.h2.decompose()
                soup.h2.replace_with('.')
        except:
            pass
        try:
            for tag in range(0,len(soup.find_all('h3'))):
                #soup.h3.decompose()
                soup.h3.replace_with('.')
        except:
            pass
        try:
            for tag in range(0,len(soup.find_all('h4'))):
                #soup.h4.decompose()
                soup.h4.replace_with('.')
        except:
            pass
        page_text = soup.article
        list_item_counter = len(page_text.find_all('li'))
        try:
            page_text.find('aside', {"class":"col-md-3"}).decompose()
        except:
            pass
        try:
            page_text.find('span', {"class":"date-display-single"}).decompose()
        except:
            pass
        try:
            page_text.find('div', {"class":"dhs-iiyl--block__wrapper rs_skip"}).decompose()
        except:
            pass
        Listen_pos = page_text.text.find('\n  Listen   \n')
        main_text = page_text.text[page_text.text.find('\n  Listen   \n')+13:page_text.text.find('Was this page useful')]
        clean_text = main_text.encode('utf-8').replace('','').replace('\xe2\x80\x99','\'').replace('\xe2\x80\x93','-').replace('\xc2\xa0',' ')#.replace('\n','')#.replace('\n\n','. ').replace('\n',',').replace('. ,','').replace(':.',': ').replace('..','. ').replace('.,','. ').replace(' . .','')
        for i in range(0,10):
            if clean_text[i].isupper():
                clean_text = clean_text[i:]
                clean_text = clean_text.replace('\xcc\x81','').replace(' \xc3\xb7 ','/').replace('\xe2\x80\x98','\'').replace('\xc2\xba',' degrees ').replace('\xe2\x84\xa2','-').replace('\xe2\x80\xa2','').replace('. ,','.').replace('. .','.')
                break
        text = re.sub( r'([a-zA-Z])\n([a-zA-Z])', r'\1; \2',clean_text)
        for k in range(0,2):
            text = re.sub( r'([a-zA-Z]);([a-zA-Z])', r'\1; \2',text)
            text = re.sub( r'(\n)([a-zA-Z])', r'\1 \2',text)
            text = re.sub( r'([a-z])([A-Z])', r'\1. \2',text)
            text = re.sub( r'([0-9])([A-Z])', r'\1 \2',text)
            text = re.sub( r'([0-9])([a-z])', r'\1 \2',text)
            text = re.sub( r'([A-Z])([0-9])', r'\1 \2',text)
            text = re.sub( r'([a-z])([0-9])', r'\1 \2',text)
            text = re.sub( r'(\))([a-zA-Z])', r'\1 \2',text)
        text = text.replace('\n','') + ' ' + ' '.join(list_of_text_from_tables) + ' List Items: '+ str(list_item_counter) + ' URL: ' + str(doc)
        ##############
        #TEXT STORAGE#
        ##############
        #Section = ''
        #Section = Centrelink_classifier(link)
        #if Section in folders:
        #    for_saving = 'C:/Python27/Scripts/Centrelink/The Final Scrape/Centrelink March 11/Folders/'+str(Section).capitalize() + '/' + title.replace('/','').replace('?','').replace(':','') + ' #' + str(row) + '.txt'
        #else:
        #    for_saving = 'C:/Python27/Scripts/Centrelink/The Final Scrape/Centrelink March 11/Folders/Other/'+ title.replace('/','').replace('?','').replace(':','') + ' #' + str(row) + '.txt'
        #with open(for_saving, 'wb') as f:
        #    f.write(str(text))
        #for_saving2 = 'C:/Python27/Scripts/Centrelink/The Final Scrape/Centrelink March 11/Text/' + title.replace('/','').replace('?','').replace(':','') + ' #' + str(row) + '.txt'
        #with open(for_saving2, 'wb') as f:
        #    f.write(str(text[:text.index('List Items:')]))
        ################
        #TEXT CRUNCHING#
        ################
        text = text[:text.index('List Items:')]
        if len(text) > 1:
            punc_free_text = text
            punc_free_text = remove_non_ascii(punc_free_text)
            for thing in Punctuation: 
                punc_free_text = punc_free_text.replace('\'','').replace(thing,' ').replace('  ',' ')
            words_list = punc_free_text.split()
            tokens = nltk.word_tokenize(punc_free_text)
            tagged = nltk.pos_tag(tokens)
            adjs_1_list  = []; adjs_1  = 0; adjs_2_list  = []; adjs_2  = 0; adjs_3_list  = []; adjs_3  = 0;
            advbs_1_list = []; advbs_1 = 0; advbs_2_list = []; advbs_2 = 0; advbs_3_list = []; advbs_3 = 0;
            deter_1_list = []; deter_1 = 0; deter_2_list = []; deter_2 = 0; deter_3_list = []; deter_3 = 0;
            inter_1_list = []; inter_1 = 0; inter_2_list = []; inter_2 = 0; inter_3_list = []; inter_3 = 0;
            nouns_1_list = []; nouns_1 = 0; nouns_2_list = []; nouns_2 = 0; nouns_3_list = []; nouns_3 = 0;
            preps_1_list = []; preps_1 = 0; preps_2_list = []; preps_2 = 0; preps_3_list = []; preps_3 = 0;
            pronn_1_list = []; pronn_1 = 0; pronn_2_list = []; pronn_2 = 0; pronn_3_list = []; pronn_3 = 0;
            verbs_1_list = []; verbs_1 = 0; verbs_2_list = []; verbs_2 = 0; verbs_3_list = []; verbs_3 = 0;
            proper_nouns = [];
            for i in range(0,len(tagged)):
                werd = tagged[i]
                fixer = 0
                if werd[0].lower() in syllable_fix_list_plus:
                    fixer = 1
                if werd[0].lower() in syllable_fix_list_minus:
                    fixer = -1
                if werd[1] in adjectives:
                    if count_syllables(werd[0])+fixer == 1:
                        adjs_1_list.append(werd[0].lower())
                        adjs_1 += 1
                    if count_syllables(werd[0])+fixer == 2:
                        adjs_2_list.append(werd[0].lower())
                        adjs_2 += 1
                    if count_syllables(werd[0])+fixer >= 3:
                        adjs_3_list.append(werd[0].lower())
                        adjs_3 += 1
                if werd[1] in adverbs:
                    if count_syllables(werd[0])+fixer == 1:
                        advbs_1_list.append(werd[0].lower())
                        advbs_1 += 1
                    if count_syllables(werd[0])+fixer == 2:
                        advbs_2_list.append(werd[0].lower())
                        advbs_2 += 1
                    if count_syllables(werd[0])+fixer >= 3:
                        advbs_3_list.append(werd[0].lower())
                        advbs_3 += 1
                if werd[1] in determiners:
                    if count_syllables(werd[0])+fixer == 1:
                        deter_1_list.append(werd[0].lower())
                        deter_1 += 1
                    if count_syllables(werd[0])+fixer == 2:
                        deter_2_list.append(werd[0].lower())
                        deter_2 += 1
                    if count_syllables(werd[0])+fixer >= 3:
                        deter_3_list.append(werd[0].lower())
                        deter_3 += 1
                if werd[1] in interjections:
                    if count_syllables(werd[0])+fixer == 1:
                        inter_1_list.append(werd[0].lower())
                        inter_1 += 1
                    if count_syllables(werd[0])+fixer == 2:
                        inter_2_list.append(werd[0].lower())
                        inter_2 += 1
                    if count_syllables(werd[0])+fixer >= 3:
                        inter_3_list.append(werd[0].lower())
                        inter_3 += 1
                if werd[1] in nouns:
                    if count_syllables(werd[0])+fixer == 1:
                        nouns_1_list.append(werd[0].lower())
                        nouns_1 += 1
                    if count_syllables(werd[0])+fixer == 2:
                        nouns_2_list.append(werd[0].lower())
                        nouns_2 += 1
                    if count_syllables(werd[0])+fixer >= 3:
                        nouns_3_list.append(werd[0].lower())
                        nouns_3 += 1
                if werd[1] in prepositions:
                    if count_syllables(werd[0])+fixer == 1:
                        preps_1_list.append(werd[0].lower())
                        preps_1 += 1
                    if count_syllables(werd[0])+fixer == 2:
                        preps_2_list.append(werd[0].lower())
                        preps_2 += 1
                    if count_syllables(werd[0])+fixer >= 3:
                        preps_3_list.append(werd[0].lower())
                        preps_3 += 1
                if werd[1] in pronouns:
                    if count_syllables(werd[0])+fixer == 1:
                        pronn_1_list.append(werd[0].lower())
                        pronn_1 += 1
                    if count_syllables(werd[0])+fixer == 2:
                        pronn_2_list.append(werd[0].lower())
                        pronn_2 += 1
                    if count_syllables(werd[0])+fixer >= 3:
                        pronn_3_list.append(werd[0].lower())
                        pronn_3 += 1
                if werd[1] in verbs:
                    if count_syllables(werd[0])+fixer == 1:
                        verbs_1_list.append(werd[0].lower())
                        verbs_1 += 1
                    if count_syllables(werd[0])+fixer == 2:
                        verbs_2_list.append(werd[0].lower())
                        verbs_2 += 1
                    if count_syllables(werd[0])+fixer >= 3:
                        verbs_3_list.append(werd[0].lower())
                        verbs_3 += 1
                if werd[1] in proper_nouns:
                    proper_nouns.append(werd[0].lower())
            try:
                texts = TextBlob(text)
                sentence_count = len(texts.sentences)
            except:
                texts = TextBlob(remove_non_ascii(text))
                sentence_count = len(texts.sentences)
            Pronoun_totaller = 0
            Pronoun_listing = []
            Restricted_Pronoun_totaller = []
            sentence_lengths = []
            words_list = punc_free_text.lower().split()
            wordCounts = Counter(words_list)
            word_count = len(punc_free_text.lower().split()) #LINE 30
            character_count = len(text)
            for word in Pronouns:
                Pronoun_totaller = Pronoun_totaller + wordCounts[word]
            try:
                Pronoun_ratio = float(Pronoun_totaller)/float(word_count)
            except:
                Pronoun_ratio = 0
            if Pronoun_ratio > .05:
                PR_ind = 'High'
            else:
                PR_ind = 'Low'
            text_syllables = []; single_syllable = []; double_syllable = []; triple_plus_syllable = []; one_syllable = 0; two_syllable = 0; three_plus = 0; three_plus_count = 0
            cohesive_counter = 0
            for werd in words_list:
                fixer = 0
                if werd in syllable_fix_list_plus:
                    fixer = 1
                if werd in syllable_fix_list_minus:
                    fixer = -1
                text_syllables.append(count_syllables(werd)+fixer)
                if count_syllables(werd)+fixer == 1:
                    single_syllable.append(werd.lower()) #LINE 40
                    one_syllable += 1
                if count_syllables(werd)+fixer == 2:
                    double_syllable.append(werd.lower())
                    two_syllable += 2
                if count_syllables(werd)+fixer >= 3: 
                    triple_plus_syllable.append(werd.lower())
                    three_plus_count += count_syllables(werd.lower()) #sum of syllables 
                if werd.lower() in Pronouns:
                    Pronoun_listing.append(werd.lower())
                if werd in cohesive_words:
                    cohesive_counter += 1
            total_syllables = sum(text_syllables)
            single_percentage = float(one_syllable)/float(total_syllables)
            double_percentage = float(two_syllable)/float(total_syllables)
            triple_percentage = float(three_plus_count)/float(total_syllables)
            syllables = str('S,'*one_syllable + 'D,'*two_syllable + 'T,'*three_plus_count)[:-1]
            average_syllables = float(total_syllables)/float(word_count)                     
            average_characters_per_word = float(character_count)/float(word_count)
            unique_word_count = len(Counter(punc_free_text.split()))
            unique_word_percentage = 100*(float(unique_word_count)/float(word_count))
            avg_words_per_sentence = float(word_count)/float(sentence_count)
            repeat_words = len([x for x in Counter(words_list).values() if x!=1]) 
            repeated_words_percentage = 100*(float(repeat_words)/float(word_count))
            FRE_result = textstat.flesch_reading_ease(text)
            if 0 <= FRE_result < 110: 
                FRE_class = FRE[int(math.ceil(FRE_result))] #textstat.flesch_reading_ease - Classification
            else:
                FRE_class = 'Very Confusing'
            FRE_R_result = 206.835 - 1.015*(np.median(sentence_lengths)) - 84.6*(np.median(text_syllables)) #ROBUST FRE
            if 0 <= FRE_R_result < 110: 
                FRE_R_class = FRE_R[int(math.ceil(FRE_R_result))] #textstat.flesch_reading_ease - Classification
            else:
                FRE_R_class = 'Very Confusing'
            SMOG_result = textstat.smog_index(text) #https://www.webpagefx.com/tools/read-able/smog-index.html
            if 0 < SMOG_result < 50:
                SMOG_class = SMOG[int(math.ceil(SMOG_result))]
            else: 
                SMOG_class = 'Unknown'
            KINCAID_result = textstat.flesch_kincaid_grade(text) #LINE 70
            #textstat.flesch_kincaid_grade - Classification
            COLEMAN_result = textstat.coleman_liau_index(text)
            if 0 <= COLEMAN_result < 50:
                COLEMAN_class = COLEMAN[int(math.ceil(COLEMAN_result))]
            else:
                COLEMAN_class = '1'
            ARI_result = textstat.automated_readability_index(text)
            if 0 <= ARI_result < 50: 
                ARI_class = ARI[int(math.ceil(ARI_result))]
                ARI_Age = ARI_age[int(math.ceil(ARI_result))] #LINE 80
            else:
                ARI_class = 'Kindergarten'
            DALE_result = textstat.dale_chall_readability_score(text)
            if 0 <= DALE_result < 10:
                DALE_class = DALE[int(math.ceil(DALE_result))]
            else:
                DALE_class = '4th grade or lower'
            #difficult_word_count = textstat.difficult_words(text)
            LINSEAR_result = textstat.linsear_write_formula(text)
            #textstat.linsear_write_formula - Classification
            GUNNING_result = textstat.gunning_fog(text)
            if 0 <= GUNNING_result < 50:
                GUNNING_class = GUNNING[int(math.ceil(GUNNING_result))]
            else: 
                GUNNING_class = 'None'
            STANDARD_result = textstat.text_standard(text)
            ws.cell(row=row, column=1, value= 'Centrelink') #Organisation
            ws.cell(row=row, column=2, value= title)  #Title/Article Name
            ws.cell(row=row, column=3, value= Section) #Section
            ws.cell(row=row, column=4, value= text)    #Text
            ws.cell(row=row, column=5, value= doc.replace('_','/').replace('.txt','')) #URL
            ws.cell(row=row, column=6, value= doc.replace('www.ndis.gov.au_','').replace('.txt','')) #Partial URL
            #ws.cell(row=row, column=7, value= ) #Intended audience
            ws.cell(row=row, column=8, value= FRE_result) #Flesch Reading Ease numerical score
            ws.cell(row=row, column=9, value= FRE_class) #Flesch Reading Ease general score
            ws.cell(row=row, column=10, value= GUNNING_result) #Gunning Fog numerical score
            ws.cell(row=row, column=11, value= GUNNING_class) #Gunning Fog general score
            ws.cell(row=row, column=12, value= KINCAID_result) #Flesch-Kincaid Grade Level detail
            #ws.cell(row=row, column=13, value= ) #Flesch-Kincaid Grade Level general
            ws.cell(row=row, column=14, value= COLEMAN_result) #Coleman-Liau Index
            ws.cell(row=row, column=15, value= COLEMAN_class) #Coleman-Liau Grade level
            ws.cell(row=row, column=16, value= SMOG_result) #SMOG Index
            ws.cell(row=row, column=17, value= SMOG_class) #SMOG grade level
            ws.cell(row=row, column=18, value= ARI_result) #Automated Readability Index
            ws.cell(row=row, column=19, value= ARI_Age) #Automated Readability Age Span
            ws.cell(row=row, column=20, value= ARI_class) #Automated Readability Grade level
            ws.cell(row=row, column=21, value= LINSEAR_result) #Linsear Write Formula
            #ws.cell(row=row, column=22, value= ) #Linsear Write grade level
            #ws.cell(row=row, column=23, value= ) #Readability Consensus
            ws.cell(row=row, column=24, value= STANDARD_result ) #Based on 8 readability formulas, Grade Level
            #ws.cell(row=row, column=25, value= ) #Reading Level
            #ws.cell(row=row, column=26, value= ) #Readers Age
            #ws.cell(row=row, column=27, value= ) #Readers grade level
            ws.cell(row=row, column=28, value= word_count) #Total # of words
            ws.cell(row=row, column=29, value= unique_word_count) #Total # of unique words
            ws.cell(row=row, column=30, value= unique_word_percentage) #unique words as % of total text
            ws.cell(row=row, column=31, value= repeat_words) #Total # of repeat words
            ws.cell(row=row, column=32, value= repeated_words_percentage) #repeat words as % of total text
            ws.cell(row=row, column=33, value= avg_words_per_sentence) #Average # of words per sentence
            ws.cell(row=row, column=34, value= sentence_count) #Total # of sentences
            ws.cell(row=row, column=35, value= character_count) #Total # of characters
            ws.cell(row=row, column=36, value= average_characters_per_word) #Average # of characters per word
            ws.cell(row=row, column=37, value= average_syllables) #Average # of syllables per word
            ws.cell(row=row, column=38, value= total_syllables) #Total syllables in text
            ws.cell(row=row, column=39, value= len(double_syllable)) #Total # of words with double syllables
            ws.cell(row=row, column=40, value= double_percentage) #Percent of double syllables in text
            ws.cell(row=row, column=41, value= len(single_syllable)) #Total # of words with single syllables
            ws.cell(row=row, column=42, value= single_percentage) #Percent of single syllables in text
            ws.cell(row=row, column=43, value= len(triple_plus_syllable)) #Total # of words with 3+ syllables
            ws.cell(row=row, column=44, value= triple_percentage) #Percent of 3+ syllables in text
            ws.cell(row=row, column=45, value= three_plus_count) #Count of the number of syllables in words with 3 or more of them
            ws.cell(row=row, column=46, value= ' '.join(single_syllable)) #Monosyllabic words
            ws.cell(row=row, column=47, value= ' '.join(double_syllable)) #Duosyllabic words
            ws.cell(row=row, column=48, value= ' '.join(triple_plus_syllable)) #Polysyllabic words 
            ws.cell(row=row, column=49, value= Pronoun_totaller) #Pronoun Counter
            ws.cell(row=row, column=50, value= Counter(words_list)['I']) #
            ws.cell(row=row, column=51, value= Counter(words_list)['me'])
            ws.cell(row=row, column=52, value= Counter(words_list)['my'])
            ws.cell(row=row, column=53, value= Counter(words_list)['mine'])
            ws.cell(row=row, column=54, value= Counter(words_list)['myself'])
            ws.cell(row=row, column=55, value= Counter(words_list)['we'])
            ws.cell(row=row, column=56, value= Counter(words_list)['us'])
            ws.cell(row=row, column=57, value= Counter(words_list)['our'])
            ws.cell(row=row, column=58, value= Counter(words_list)['ours'])
            ws.cell(row=row, column=59, value= Counter(words_list)['ourselves'])
            ws.cell(row=row, column=60, value= Counter(words_list)['you'])
            ws.cell(row=row, column=61, value= Counter(words_list)['your'])
            ws.cell(row=row, column=62, value= Counter(words_list)['yours'])
            ws.cell(row=row, column=63, value= Counter(words_list)['yourself'])
            ws.cell(row=row, column=64, value= Counter(words_list)['yourselves'])
            ws.cell(row=row, column=65, value= Counter(words_list)['he'])
            ws.cell(row=row, column=66, value= Counter(words_list)['him'])
            ws.cell(row=row, column=67, value= Counter(words_list)['his'])
            ws.cell(row=row, column=68, value= Counter(words_list)['himself'])
            ws.cell(row=row, column=69, value= Counter(words_list)['she'])
            ws.cell(row=row, column=70, value= Counter(words_list)['her'])
            ws.cell(row=row, column=71, value= Counter(words_list)['hers'])
            ws.cell(row=row, column=72, value= Counter(words_list)['herself'])
            ws.cell(row=row, column=73, value= Counter(words_list)['it'])
            ws.cell(row=row, column=74, value= Counter(words_list)['its'])
            ws.cell(row=row, column=75, value= Counter(words_list)['itself'])
            ws.cell(row=row, column=76, value= Counter(words_list)['they'])
            ws.cell(row=row, column=77, value= Counter(words_list)['them'])
            ws.cell(row=row, column=78, value= Counter(words_list)['their'])
            ws.cell(row=row, column=79, value= Counter(words_list)['theirs'])
            ws.cell(row=row, column=80, value= Counter(words_list)['themselves'])
            ws.cell(row=row, column=81, value= Pronoun_ratio)
            ws.cell(row=row, column=82, value= PR_ind)
            ws.cell(row=row, column=83, value= ' '.join(Pronoun_listing))
            #ws.cell(row=row, column=84, value= noun_count)
            #ws.cell(row=row, column=85, value= verb_count)
            #ws.cell(row=row, column=86, value= adjective_count)
            #ws.cell(row=row, column=87, value= adverb_count)
            #ws.cell(row=row, column=88, value= pronoun_count)
            #ws.cell(row=row, column=89, value= preposition_count)
            #ws.cell(row=row, column=90, value= determiner_count)
            #ws.cell(row=row, column=91, value= interjection_count)                        
            ws.cell(row=row, column=92, value= adjs_1)
            ws.cell(row=row, column=93, value= ' '.join(adjs_1_list))
            ws.cell(row=row, column=94, value= adjs_2)
            ws.cell(row=row, column=95, value= ' '.join(adjs_2_list))
            ws.cell(row=row, column=96, value= adjs_3)
            ws.cell(row=row, column=97, value= ' '.join(adjs_3_list))
            ws.cell(row=row, column=98, value= advbs_1)
            ws.cell(row=row, column=99, value= ' '.join(advbs_1_list))
            ws.cell(row=row, column=100, value= advbs_2)
            ws.cell(row=row, column=101, value= ' '.join(advbs_2_list))
            ws.cell(row=row, column=102,value= advbs_3)
            ws.cell(row=row, column=103, value= ' '.join(advbs_3_list))
            ws.cell(row=row, column=104, value= deter_1)
            ws.cell(row=row, column=105, value= ' '.join(deter_1_list))
            ws.cell(row=row, column=106, value= deter_2)
            ws.cell(row=row, column=107, value= ' '.join(deter_2_list))
            ws.cell(row=row, column=108, value= deter_3)
            ws.cell(row=row, column=109, value= ' '.join(deter_3_list))
            ws.cell(row=row, column=110, value= inter_1)
            ws.cell(row=row, column=111, value= ' '.join(inter_1_list))
            ws.cell(row=row, column=112, value= inter_2)
            ws.cell(row=row, column=113, value= ' '.join(inter_2_list))
            ws.cell(row=row, column=114, value= inter_3)
            ws.cell(row=row, column=115, value= ' '.join(inter_3_list))
            ws.cell(row=row, column=116, value= nouns_1)
            ws.cell(row=row, column=117, value= ' '.join(nouns_1_list))
            ws.cell(row=row, column=118, value= nouns_2)
            ws.cell(row=row, column=119, value= ' '.join(nouns_2_list))
            ws.cell(row=row, column=120, value= nouns_3)
            ws.cell(row=row, column=121, value= ' '.join(nouns_3_list))
            ws.cell(row=row, column=122, value= preps_1)
            ws.cell(row=row, column=123, value= ' '.join(preps_1_list))
            ws.cell(row=row, column=124, value= preps_2)
            ws.cell(row=row, column=125, value= ' '.join(preps_2_list))
            ws.cell(row=row, column=126, value= preps_3)
            ws.cell(row=row, column=127, value= ' '.join(preps_3_list))
            ws.cell(row=row, column=128, value= pronn_1)
            ws.cell(row=row, column=129, value= ' '.join(pronn_1_list))
            ws.cell(row=row, column=130, value= pronn_2)
            ws.cell(row=row, column=131, value= ' '.join(pronn_2_list))
            ws.cell(row=row, column=132, value= pronn_3)
            ws.cell(row=row, column=133, value= ' '.join(pronn_3_list))
            ws.cell(row=row, column=134, value= verbs_1)
            ws.cell(row=row, column=135, value= ' '.join(verbs_1_list))
            ws.cell(row=row, column=136, value= verbs_2)
            ws.cell(row=row, column=137, value= ' '.join(verbs_2_list))
            ws.cell(row=row, column=138, value= verbs_3)
            ws.cell(row=row, column=139, value= ' '.join(verbs_3_list))
            ws.cell(row=row, column=140, value= len(proper_nouns))
            ws.cell(row=row, column=141, value= ' '.join(proper_nouns))
            if sentence_count >= 30:
                ws.cell(row=row, column=142, value= 1)
            else:
                ws.cell(row=row, column=142, value= 0)
            ws.cell(row=row, column=143, value= cohesive_counter)
            ws.cell(row=row, column=144, value= list_item_counter) #List Item Count
            row = row + 1
   

saving = 'C:/Python27/Scripts/Centrelink/The Final Scrape/Centrelink March 11.xlsx'
wb.save(saving)

################
#Classification#
################
def Centrelink_classifier(Abridged_URL):
    if 'individuals' in Abridged_URL:
        if 'online-help' in Abridged_URL:
            Section = "Online Help"
        else:
            try:
                if '/' not in Abridged_URL.split('/centrelink')[1]:
                    Section = "Other"
            except:
                Section = 'Other'
            else:
                if 'abstudy' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Abstudy Austudy'
                elif 'austudy' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Abstudy Austudy'
                elif 'jobs-education' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Jobs Education'
                elif 'child-care' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Child Care'
                elif 'double-orphan' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Double Orphan'
                elif 'age-pension' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Pension'
                elif 'pension-' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Pension'
                elif 'pensioner' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Pension'
                elif '-pension' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Pension'                    
                elif 'isolated-children' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Isolated Children'
                elif 'assurance' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Assurance Support'
                elif 'basicscard' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Basics Card'
                elif 'bereavement' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Bereavement Allowance'
                elif 'carer' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Carer'
                elif '-centrelink' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Other Centrelink Stuff'
                elif 'centrepay' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Centrepay'
                elif 'health-card' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Health Care Card'
                elif 'health-care-card' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Health Care Card'
                elif 'crisis' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Crisis Payment'
                elif 'dad-and' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Dad and Partner Pay'
                elif 'disability' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Disability'
                elif 'early-release' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Early Release Superannuation'
                elif 'electronic-messaging' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Electronic Messaging'
                elif 'family-tax' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Family Tax Benefit'
                elif 'farm-household' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Farm or Remote Area'
                elif 'remote' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Farm or Remote Area'     
                elif 'income-management' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Income Management'
                elif 'mobility-allowance' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Mobility Allowance'
                elif 'newborn-upfront' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Newborn Upfront'
                elif 'newstart' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Newstart'
                elif 'parental-leave' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Parental Leave Pay'
                elif 'parenting-payment' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Parenting Payment'
                elif 'partner-allowance' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Partner Allowance'
                elif 'relocation' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Relocation'
                elif 'rent-assistance' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Rent Assistance'
                elif 'sickness-allowance' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Sickness Allowance'
                elif 'single-income-family' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Single Income Family'
                elif 'special-benefit' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Special Benefit'
                elif 'stillborn-baby' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Stillborn Baby Payment'
                elif 'student-start' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Student Start Loan'
                elif 'telephone-allowance' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Telephone Allowance'
                elif 'widow-allowance' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Widow Allowance'
                elif 'youth-allowance' in Abridged_URL.split('/centrelink')[1]:
                    Section = 'Youth Allowance'
                else:
                    Section = 'Other'  
    else:
        if 'about-us' in Abridged_URL:
                Section = 'About Us'
        elif 'services' in Abridged_URL:
            Section = 'Business Services'
        else:
            Section = 'Other'
    return Section
    

##################
#Syllable Counter#
##################

verbose = False
def count_syllables(word):
    vowels = ['a', 'e', 'i', 'o', 'u']
    on_vowel = False
    in_diphthong = False
    minsyl = 0
    maxsyl = 0
    lastchar = None
    word = word.lower()
    for c in word:
        is_vowel = c in vowels
        if on_vowel == None:
            on_vowel = is_vowel
        if c == 'y':
            is_vowel = not on_vowel
        if is_vowel:
            if verbose: print("%s is a vowel" % c)
            if not on_vowel:
                # We weren't on a vowel before.
                # Seeing a new vowel bumps the syllable count.
                if verbose: print("new syllable")
                minsyl += 1
                maxsyl += 1
            elif on_vowel and not in_diphthong and c != lastchar:
                # We were already in a vowel.
                # Don't increment anything except the max count,
                # and only do that once per diphthong.
                if verbose: print("%s is a diphthong" % c)
                in_diphthong = True
                maxsyl += 1
        elif verbose: print("[consonant]")
        on_vowel = is_vowel
        lastchar = c
    if word[-1] == 'e':
        minsyl -= 1
    if word[-1] == 'y' and not on_vowel:
        maxsyl += 1
    if not minsyl:
        minsyl = 1
    return minsyl#, maxsyl





